"""
Core agent loop.
1. Receives a prompt
2. Calls LLM (with tools)
3. Executes tool calls
4. Streams results back to the CLI
"""

from __future__ import annotations

import asyncio
import json
from pathlib import Path
from typing import Any, Dict, List, Optional

from kimi_code.config import Settings
from kimi_code.diff import read_lines, write_lines
from kimi_code.llm.base import Message
from kimi_code.llm.kimi import KimiProvider
from kimi_code.llm.openai import OpenAIProvider
from kimi_code.state import State
from kimi_code.tools import get_tool, list_tools


class AgentSession:
    """
    One agent invocation tied to a single prompt.
    """

    def __init__(self, settings: Settings, cwd: Path) -> None:
        self.settings = settings
        self.cwd = cwd.resolve()
        self.state = State(settings)
        self.provider = self._make_provider()
        self.messages: List[Message] = []
        self.run_id: int | None = None

    # --------------------------------------------------------------------- #
    # Provider factory
    # --------------------------------------------------------------------- #
    def _make_provider(self) -> Any:  # Provider
        if self.settings.provider == "kimi":
            if not self.settings.api_key:
                raise RuntimeError("Kimi API key missing")
            return KimiProvider(self.settings.api_key, self.settings.model)
        elif self.settings.provider == "openai":
            if not self.settings.api_key:
                raise RuntimeError("OpenAI API key missing")
            return OpenAIProvider(self.settings.api_key, self.settings.model)
        else:
            raise NotImplementedError(f"Provider {self.settings.provider}")

    # --------------------------------------------------------------------- #
    # Public API
    # --------------------------------------------------------------------- #
    async def run(self, prompt: str) -> int:
        """
        Run the agent loop until completion or max tool calls.
        Returns 0 on success, non-zero on error.
        """
        self.run_id = self.state.create_run(prompt)
        self.messages = [
            {
                "role": "system",
                "content": (
                    "You are Kimi-Code, an expert coding agent. "
                    "You MUST use the provided tools to complete tasks. "
                    "When asked to create, edit, or manipulate files, you MUST use the appropriate tools. "
                    "Never just provide instructions - always execute the required actions using tools.\n\n"
                    "Available tools:\n"
                    "- write_file: Create or overwrite a file\n"
                    "  Parameters: {\"path\": \"file/path\", \"content\": \"file contents\"}\n"
                    "- read_file: Read a file's contents\n"
                    "  Parameters: {\"path\": \"file/path\"}\n"
                    "- list_dir: List directory contents\n"
                    "  Parameters: {\"path\": \"directory/path\", \"recursive\": true/false}\n"
                    "- shell: Execute shell commands\n"
                    "  Parameters: {\"command\": \"shell command\"}\n\n"
                    "Use tool calls in this format:\n"
                    "```json\n"
                    '{"name": "tool_name", "arguments": {"params": "{\\"param\\": \\"value\\"}"}}\n'
                    "```\n"
                    "Always confirm success before finishing."
                ),
            },
            {"role": "user", "content": prompt},
        ]

        exit_code = 0
        try:
            await self._loop()
        except Exception as exc:
            self.state.add_message(self.run_id, "assistant", f"Fatal: {exc}")
            exit_code = 1
        finally:
            self.state.finish_run(self.run_id, exit_code)
            await self.provider.close()
        return exit_code

    # --------------------------------------------------------------------- #
    # Loop
    # --------------------------------------------------------------------- #
    async def _loop(self) -> None:
        tools = self._build_tools_schema()
        max_calls = self.settings.max_tool_calls
        calls = 0

        while calls < max_calls:
            calls += 1
            # Stream assistant response
            content_chunks: List[str] = []
            async for chunk in self.provider.chat(self.messages, tools):
                print(chunk, end="", flush=True)  # simple streaming
                content_chunks.append(chunk)
            content = "".join(content_chunks).strip()

            self.messages.append({"role": "assistant", "content": content})
            self.state.add_message(self.run_id, "assistant", content)

            # Detect tool calls (JSON inside triple-backticks)
            tool_calls = self._extract_tool_calls(content)
            if not tool_calls:
                # Natural language answer → done
                break

            for i, tc in enumerate(tool_calls):
                result = await self._execute_tool(tc)
                
                # For simple file creation tasks, if the tool succeeded, we're done
                if result.get("status") == "ok" and tc["name"] in ["write_file", "shell"]:
                    return  # Exit the loop early
                
                # Only continue the conversation if we need to handle errors or complex workflows
                self.messages.append(
                    {
                        "role": "user",  # Convert tool response to user message to avoid unsupported 'tool' role
                        "content": f"Tool {tc['name']} result: {json.dumps(result, ensure_ascii=False)}",
                    }
                )
                self.state.add_message(
                    self.run_id,
                    "tool",
                    json.dumps(result, ensure_ascii=False),
                    tool_call=tc,
                )

    # --------------------------------------------------------------------- #
    # Tools
    # --------------------------------------------------------------------- #
    def _build_tools_schema(self) -> List[Dict[str, Any]]:
        schemas = []
        for name, func in list_tools().items():
            # VERY simple schema: one string param "params"
            schemas.append(
                {
                    "type": "function",
                    "function": {
                        "name": name,
                        "description": func.__doc__ or "",
                        "parameters": {
                            "type": "object",
                            "properties": {
                                "params": {
                                    "type": "string",
                                    "description": "JSON-encoded parameters",
                                }
                            },
                            "required": ["params"],
                        },
                    },
                }
            )
        return schemas

    async def _execute_tool(self, tc: Dict[str, Any]) -> Dict[str, Any]:
        name = tc["name"]
        raw = tc["arguments"]["params"]
        try:
            params = json.loads(raw)
        except json.JSONDecodeError as exc:
            return {"status": "error", "message": str(exc)}

        tool_func = get_tool(name)
        if not tool_func:
            return {"status": "error", "message": f"Unknown tool {name}"}

        # Inject cwd into params if missing
        params.setdefault("cwd", str(self.cwd))

        # Run tool (all tools are async)
        try:
            return await tool_func(params)
        except Exception as exc:
            return {"status": "error", "message": str(exc)}

    # --------------------------------------------------------------------- #
    # Helpers
    # --------------------------------------------------------------------- #
    def _extract_tool_calls(self, content: str) -> List[Dict[str, Any]]:
        """
        Naïve parser: looks for
        ```json
        {"name": "...", "arguments": {...}}
        ```
        blocks.
        """
        import re

        pattern = re.compile(r"```(?:json)?\s*(\{.*?\})\s*```", re.S)
        calls = []
        for match in pattern.finditer(content):
            try:
                data = json.loads(match.group(1))
                if data.get("name") and isinstance(data.get("arguments"), dict):
                    calls.append(data)
            except Exception:
                pass
        return calls