"""
Entry-point for the `kimi` CLI (Click-based).
"""

from __future__ import annotations

import asyncio
import sys
from pathlib import Path

import click
from rich.console import Console
from rich.markdown import Markdown

from kimi_code.agent import AgentSession
from kimi_code.config import Settings

console = Console()


@click.group(invoke_without_command=True)
@click.pass_context
def main(ctx: click.Context) -> None:
    """
    Kimi-Code: an open-source Claude-Code-style agent.
    """
    if ctx.invoked_subcommand is None:
        # No subcommand provided, start interactive mode
        asyncio.run(_interactive_mode())


# --------------------------------------------------------------------------- #
# Subcommands
# --------------------------------------------------------------------------- #
@main.command("run")
@click.option(
    "--cwd",
    type=click.Path(exists=True, file_okay=False, path_type=Path),
    default=Path.cwd(),
    help="Working directory for the agent.",
)
@click.argument("prompt")
def run(cwd: Path, prompt: str) -> None:
    """Start an agent session with the given PROMPT."""
    asyncio.run(_run_agent(cwd, prompt))


@main.command("init")
@click.option(
    "--provider",
    type=click.Choice(["kimi", "openai"]),
    default="kimi",
    show_default=True,
    help="Which LLM provider to configure.",
)
@click.option("--api-key", prompt=True, hide_input=True)
def init(provider: str, api_key: str) -> None:
    """Create ~/.kimi-code/config.toml with initial settings."""
    settings = Settings()
    settings.provider = provider
    settings.api_key = api_key
    settings.config_dir.mkdir(parents=True, exist_ok=True)
    cfg_path = settings.config_dir / "config.toml"
    with cfg_path.open("w") as f:
        f.write(f'provider = "{provider}"\napi_key = "{api_key}"\n')
    console.print(f"[green]Config written to[/] {cfg_path}")


@main.command("config")
def show_config() -> None:
    """Print current effective configuration."""
    settings = Settings.load()
    console.print_json(settings.model_dump_json(indent=2))


# --------------------------------------------------------------------------- #
# Interactive Mode
# --------------------------------------------------------------------------- #
async def _interactive_mode() -> None:
    """Interactive REPL-style mode for kimi-code."""
    settings = Settings.load()
    if settings.api_key is None:
        console.print(
            "[bold red]API key not set.[/] Run: [cyan]kimi init --provider <name>[/]"
        )
        sys.exit(1)

    cwd = Path.cwd()
    
    # Welcome message
    console.print()
    console.print("[bold blue]╭──────────────────────────────────────────────╮[/]")
    console.print("[bold blue]│[/]  [bold white]██╗  ██╗██╗███╗   ███╗██╗[/]  [bold cyan]  ██████╗ ██████╗ ██████╗ ███████╗[/]  [bold blue]│[/]")
    console.print("[bold blue]│[/]  [bold white]██║ ██╔╝██║████╗ ████║██║[/]  [bold cyan]██╔════╝██╔═══██╗██╔══██╗██╔════╝[/]  [bold blue]│[/]")
    console.print("[bold blue]│[/]  [bold white]█████╔╝ ██║██╔████╔██║██║[/]  [bold cyan]██║     ██║   ██║██║  ██║█████╗[/]    [bold blue]│[/]")
    console.print("[bold blue]│[/]  [bold white]██╔═██╗ ██║██║╚██╔╝██║██║[/]  [bold cyan]██║     ██║   ██║██║  ██║██╔══╝[/]    [bold blue]│[/]")
    console.print("[bold blue]│[/]  [bold white]██║  ██╗██║██║ ╚═╝ ██║██║[/]  [bold cyan]╚██████╗╚██████╔╝██████╔╝███████╗[/]  [bold blue]│[/]")
    console.print("[bold blue]│[/]  [bold white]╚═╝  ╚═╝╚═╝╚═╝     ╚═╝╚═╝[/]  [bold cyan] ╚═════╝ ╚═════╝ ╚═════╝ ╚══════╝[/]  [bold blue]│[/]")
    console.print("[bold blue]╰──────────────────────────────────────────────╯[/]")
    console.print()
    console.print(f"[dim]📁 Working directory: {cwd}[/]")
    console.print(f"[dim]🤖 Provider: {settings.provider} | Model: {settings.model}[/]")
    console.print(f"[dim]🔥 API: Moonshot AI | Ready to code![/]")
    console.print()
    console.print("[dim]Type your requests below. Press [/][bold]Ctrl+C[/][dim] to exit or [/][bold]help[/][dim] for commands.[/]")
    console.print()

    try:
        while True:
            try:
                # Get user input
                prompt = console.input("[bold green]🔥 kimi >[/] ")
                if not prompt.strip():
                    continue
                
                # Handle special commands
                if prompt.strip().lower() in ['exit', 'quit', 'bye']:
                    console.print("[dim]Goodbye! 👋[/]")
                    break
                elif prompt.strip().lower() in ['help', '?']:
                    _show_interactive_help()
                    continue
                elif prompt.strip().lower() == 'clear':
                    console.clear()
                    continue
                
                # Run the agent
                console.print()
                session = AgentSession(settings, cwd)
                exit_code = await session.run(prompt.strip())
                
                if exit_code == 0:
                    console.print("\n[dim green]✅ Task completed successfully[/]")
                else:
                    console.print(f"\n[dim red]❌ Task failed (exit code: {exit_code})[/]")
                console.print()
                
            except KeyboardInterrupt:
                console.print("\n[dim]Goodbye! 👋[/]")
                break
            except EOFError:
                console.print("\n[dim]Goodbye! 👋[/]")
                break
            
    except Exception as e:
        console.print(f"\n[bold red]Error:[/] {e}")
        sys.exit(1)


def _show_interactive_help() -> None:
    """Show help for interactive mode."""
    console.print()
    console.print("[bold]Interactive Commands:[/]")
    console.print("  [cyan]help, ?[/]     - Show this help")
    console.print("  [cyan]clear[/]       - Clear the screen") 
    console.print("  [cyan]exit, quit[/]  - Exit interactive mode")
    console.print()
    console.print("[bold]Examples:[/]")
    console.print("  [dim]create a Python file called hello.py[/]")
    console.print("  [dim]show me the contents of README.md[/]")
    console.print("  [dim]run the tests in the current directory[/]")
    console.print()


# --------------------------------------------------------------------------- #
# Helpers
# --------------------------------------------------------------------------- #
async def _run_agent(cwd: Path, prompt: str) -> None:
    settings = Settings.load()
    if settings.api_key is None:
        console.print(
            "[bold red]API key not set.[/] Run: [cyan]kimi init --provider <name>[/]"
        )
        sys.exit(1)

    session = AgentSession(settings, cwd)
    console.print(Markdown(f"**Prompt:** {prompt}"))
    exit_code = await session.run(prompt)
    console.print(f"\n[dim]Agent finished with exit code {exit_code}[/]")