"""
Global configuration for kimi-code.
Uses pydantic-settings so values can come from:
  - ~/.kimi-code/config.toml
  - environment variables (KIMI_CODE_*)
  - CLI flags (future)
"""

from __future__ import annotations

import toml
from pathlib import Path
from typing import Literal, Any, Dict, Tuple, Type

from pydantic import Field
from pydantic_settings import BaseSettings, SettingsConfigDict, PydanticBaseSettingsSource

DEFAULT_CONFIG_DIR = Path.home() / ".kimi-code"
DEFAULT_CONFIG_FILE = DEFAULT_CONFIG_DIR / "config.toml"
DEFAULT_STATE_DB = DEFAULT_CONFIG_DIR / "state.db"


class TomlConfigSettingsSource(PydanticBaseSettingsSource):
    """Custom settings source to load from TOML file."""
    
    def get_field_value(
        self, field_info: Any, field_name: str
    ) -> Tuple[Any, str, bool]:
        # This method is required but we implement prepare instead
        return None, field_name, False

    def prepare_field_value(
        self, field_name: str, field_info: Any, value: Any, value_is_complex: bool
    ) -> Any:
        return value

    def __call__(self) -> Dict[str, Any]:
        toml_file = self.config.get('toml_file')
        if toml_file and Path(toml_file).exists():
            try:
                return toml.load(toml_file)
            except Exception:
                return {}
        return {}


class Settings(BaseSettings):
    provider: Literal["kimi", "openai"] = Field(default="kimi")
    api_key: str | None = Field(default=None)
    model: str = Field(default="moonshot-v1-8k")
    max_tokens: int = Field(default=8000, ge=1)
    temperature: float = Field(default=0.0, ge=0.0, le=2.0)
    max_tool_calls: int = Field(default=10, ge=1)
    shell_timeout: int = Field(default=30, ge=1)
    config_dir: Path = Field(default=DEFAULT_CONFIG_DIR)
    state_db: Path = Field(default=DEFAULT_STATE_DB)

    model_config = SettingsConfigDict(
        env_prefix="KIMI_CODE_",
        toml_file=str(DEFAULT_CONFIG_FILE),
    )

    def __init__(self, **kwargs) -> None:  # noqa: D107
        super().__init__(**kwargs)
        self.config_dir.mkdir(parents=True, exist_ok=True)

    @classmethod
    def settings_customise_sources(
        cls,
        settings_cls: Type[BaseSettings],
        init_settings: PydanticBaseSettingsSource,
        env_settings: PydanticBaseSettingsSource,
        dotenv_settings: PydanticBaseSettingsSource,
        file_secret_settings: PydanticBaseSettingsSource,
    ) -> Tuple[PydanticBaseSettingsSource, ...]:
        return (
            init_settings,
            env_settings,
            TomlConfigSettingsSource(settings_cls),
            file_secret_settings,
        )

    @classmethod
    def load(cls) -> "Settings":
        # Force pydantic-settings to read the TOML file
        return cls()