"""
Diff utilities:
  - compute unified diff strings
  - apply patches safely
  - create LSP-style TextEdits (future)
"""

from __future__ import annotations

import difflib
from pathlib import Path
from typing import Iterable, List, Tuple


def make_diff(
    old_lines: Iterable[str],
    new_lines: Iterable[str],
    fromfile: str = "old",
    tofile: str = "new",
) -> str:
    """Return a unified diff string."""
    return "".join(
        difflib.unified_diff(
            old_lines,
            new_lines,
            fromfile=fromfile,
            tofile=tofile,
            lineterm="",
        )
    )


def apply_patch(
    original_text: str, patch_text: str, fuzz: int = 0
) -> Tuple[str, List[str]]:
    """
    Apply a unified-diff patch to *original_text*.
    Returns (new_text, list_of_warnings).
    Uses the `diff_match_patch` library internally for robustness.
    """
    import diff_match_patch  # type: ignore

    dmp = diff_match_patch.diff_match_patch()
    dmp.Patch_Margin = 4
    patches = dmp.patch_fromText(patch_text)
    new_text, results = dmp.patch_apply(patches, original_text)

    warnings: List[str] = []
    for i, ok in enumerate(results):
        if not ok:
            warnings.append(f"Failed to apply hunk {i + 1}")

    return new_text, warnings


def read_lines(path: Path) -> List[str]:
    """Read file as list of lines (with trailing newlines)."""
    with path.open("rt", encoding="utf-8") as f:
        return f.readlines()


def write_lines(path: Path, lines: Iterable[str]) -> None:
    """Write list of lines back to file."""
    with path.open("wt", encoding="utf-8") as f:
        f.writelines(lines)