"""
Kimi provider implementation.
Uses the public REST endpoint:
  POST https://api.moonshot.cn/v1/chat/completions
Streaming via Server-Sent Events (SSE).
"""

from __future__ import annotations

import json
from typing import Any, Dict, List, Optional

import httpx

from .base import Message, Provider, ToolCall

KIMI_BASE_URL = "https://api.moonshot.cn/v1"


class KimiProvider(Provider):
    def __init__(self, api_key: str, model: str = "k1") -> None:
        self.api_key = api_key
        self.model = model
        self._client = httpx.AsyncClient(
            base_url=KIMI_BASE_URL,
            headers={"Authorization": f"Bearer {api_key}"},
            timeout=httpx.Timeout(60.0),
        )

    async def chat(
        self,
        messages: List[Message],
        tools: Optional[List[Dict[str, Any]]] = None,
    ) -> Any:  # AsyncIterator[str] but we yield raw delta text
        url = "/chat/completions"
        payload: Dict[str, Any] = {
            "model": self.model,
            "messages": messages,
            "stream": False,  # Disable streaming for now
            "temperature": 0.0,
        }
        if tools:
            payload["tools"] = tools
            payload["tool_choice"] = "auto"

        # Non-streaming response
        resp = await self._client.post(url, json=payload)
        resp.raise_for_status()
        
        result = resp.json()
        if "choices" in result and len(result["choices"]) > 0:
            content = result["choices"][0]["message"]["content"]
            # Yield the content character by character to simulate streaming
            for char in content:
                yield char

    async def close(self) -> None:
        await self._client.aclose()