"""
OpenAI (and OpenAI-compatible) provider implementation.
Streams via Server-Sent Events.
"""

from __future__ import annotations

import json
from typing import Any, Dict, List, Optional

import httpx

from .base import Message, Provider

OpenAI_BASE_URL = "https://api.openai.com/v1"


class OpenAIProvider(Provider):
    def __init__(
        self,
        api_key: str,
        model: str = "gpt-4o-mini",
        base_url: str = OpenAI_BASE_URL,
    ) -> None:
        self.api_key = api_key
        self.model = model
        self.base_url = base_url
        self._client = httpx.AsyncClient(
            base_url=base_url,
            headers={"Authorization": f"Bearer {api_key}"},
            timeout=httpx.Timeout(60.0),
        )

    async def chat(
        self,
        messages: List[Message],
        tools: Optional[List[Dict[str, Any]]] = None,
    ) -> Any:  # AsyncIterator[str]
        url = "/chat/completions"
        payload: Dict[str, Any] = {
            "model": self.model,
            "messages": messages,
            "stream": True,
            "temperature": 0.0,
        }
        if tools:
            payload["tools"] = tools
            payload["tool_choice"] = "auto"

        async with self._client.stream("POST", url, json=payload) as resp:
            resp.raise_for_status()
            async for line in resp.aiter_lines():
                if not line.startswith("data: "):
                    continue
                data = line[6:].strip()
                if data == "[DONE]":
                    break
                try:
                    chunk = json.loads(data)
                    delta = chunk["choices"][0]["delta"]
                    if "content" in delta and delta["content"]:
                        yield delta["content"]
                except (KeyError, json.JSONDecodeError):
                    continue

    async def close(self) -> None:
        await self._client.aclose()