"""
Lightweight persistence layer backed by SQLite.

Tables:
  runs           – one row per agent invocation.
  messages       – user / assistant / tool messages for each run.
  checkpoints    – file snapshots for undo/redo.
"""

from __future__ import annotations

import json
import sqlite3
from datetime import datetime
from pathlib import Path
from typing import Any, List, Literal

from kimi_code.config import Settings

MessageRole = Literal["system", "user", "assistant", "tool"]


class State:
    """Thin wrapper around SQLite."""

    def __init__(self, settings: Settings) -> None:
        self.db_path: Path = settings.state_db
        self.db_path.parent.mkdir(parents=True, exist_ok=True)
        self._init_schema()

    # ------------------------------------------------------------------ #
    # Schema
    # ------------------------------------------------------------------ #
    def _init_schema(self) -> None:
        with sqlite3.connect(self.db_path) as conn:
            conn.executescript(
                """
                CREATE TABLE IF NOT EXISTS runs (
                    id            INTEGER PRIMARY KEY AUTOINCREMENT,
                    started_at    TEXT NOT NULL,
                    finished_at   TEXT,
                    prompt        TEXT NOT NULL,
                    exit_code     INTEGER
                );

                CREATE TABLE IF NOT EXISTS messages (
                    id        INTEGER PRIMARY KEY AUTOINCREMENT,
                    run_id    INTEGER REFERENCES runs(id),
                    role      TEXT NOT NULL,
                    content   TEXT NOT NULL,
                    tool_call JSON,
                    timestamp TEXT NOT NULL
                );

                CREATE TABLE IF NOT EXISTS checkpoints (
                    id      INTEGER PRIMARY KEY AUTOINCREMENT,
                    run_id  INTEGER REFERENCES runs(id),
                    path    TEXT NOT NULL,
                    blob    BLOB NOT NULL,
                    created_at TEXT NOT NULL
                );
                """
            )

    # ------------------------------------------------------------------ #
    # Runs
    # ------------------------------------------------------------------ #
    def create_run(self, prompt: str) -> int:
        with sqlite3.connect(self.db_path) as conn:
            cur = conn.execute(
                "INSERT INTO runs (started_at, prompt) VALUES (?, ?)",
                (datetime.utcnow().isoformat(), prompt),
            )
            return cur.lastrowid  # type: ignore[return-value]

    def finish_run(self, run_id: int, exit_code: int) -> None:
        with sqlite3.connect(self.db_path) as conn:
            conn.execute(
                "UPDATE runs SET finished_at = ?, exit_code = ? WHERE id = ?",
                (datetime.utcnow().isoformat(), exit_code, run_id),
            )

    # ------------------------------------------------------------------ #
    # Messages
    # ------------------------------------------------------------------ #
    def add_message(
        self,
        run_id: int,
        role: MessageRole,
        content: str,
        tool_call: dict[str, Any] | None = None,
    ) -> None:
        with sqlite3.connect(self.db_path) as conn:
            conn.execute(
                "INSERT INTO messages (run_id, role, content, tool_call, timestamp) "
                "VALUES (?, ?, ?, ?, ?)",
                (
                    run_id,
                    role,
                    content,
                    json.dumps(tool_call) if tool_call else None,
                    datetime.utcnow().isoformat(),
                ),
            )

    def list_messages(self, run_id: int) -> List[dict[str, Any]]:
        with sqlite3.connect(self.db_path) as conn:
            cur = conn.execute(
                "SELECT role, content, tool_call FROM messages WHERE run_id = ? ORDER BY id",
                (run_id,),
            )
            rows = cur.fetchall()
        return [
            {"role": r[0], "content": r[1], "tool_call": json.loads(r[2]) if r[2] else None}
            for r in rows
        ]

    # ------------------------------------------------------------------ #
    # Checkpoints
    # ------------------------------------------------------------------ #
    def save_checkpoint(self, run_id: int, path: Path, content: bytes) -> None:
        with sqlite3.connect(self.db_path) as conn:
            conn.execute(
                "INSERT INTO checkpoints (run_id, path, blob, created_at) VALUES (?, ?, ?, ?)",
                (run_id, str(path), content, datetime.utcnow().isoformat()),
            )

    def load_checkpoint(self, run_id: int, path: Path) -> bytes | None:
        with sqlite3.connect(self.db_path) as conn:
            cur = conn.execute(
                "SELECT blob FROM checkpoints WHERE run_id = ? AND path = ? "
                "ORDER BY id DESC LIMIT 1",
                (run_id, str(path)),
            )
            row = cur.fetchone()
            return row[0] if row else None