"""
Tool registry – each tool is a simple async callable that receives
a single JSON-serializable dict and returns a dict[str, Any].
"""

from __future__ import annotations

from typing import Any, Callable, Dict

ToolFunc = Callable[[Dict[str, Any]], Any]

_REGISTRY: Dict[str, ToolFunc] = {}


def register(name: str) -> Callable[[ToolFunc], ToolFunc]:
    """Decorator to register a tool under a given name."""

    def decorator(func: ToolFunc) -> ToolFunc:
        _REGISTRY[name] = func
        return func

    return decorator


def get_tool(name: str) -> ToolFunc | None:
    """Lookup a tool by name."""
    return _REGISTRY.get(name)


def list_tools() -> Dict[str, ToolFunc]:
    """Return a copy of the registry."""
    return _REGISTRY.copy()


# Import all tool modules to register them
from . import fs
from . import git
from . import lsp
from . import shell