"""
File-system tools: read_file, write_file, list_dir.
"""

from __future__ import annotations

import os
from pathlib import Path
from typing import Any, Dict, List

from kimi_code.tools import register
from kimi_code.diff import apply_patch, make_diff, read_lines, write_lines


@register("read_file")
async def read_file(params: Dict[str, Any]) -> Dict[str, Any]:
    path = Path(params["path"]).expanduser().resolve()
    if not path.is_file():
        return {"status": "error", "message": f"{path} is not a file"}
    try:
        lines = read_lines(path)
    except Exception as exc:
        return {"status": "error", "message": str(exc)}
    return {"status": "ok", "lines": lines}


@register("write_file")
async def write_file(params: Dict[str, Any]) -> Dict[str, Any]:
    path = Path(params["path"]).expanduser().resolve()
    if "patch" in params:
        # Patch mode
        if not path.exists():
            return {"status": "error", "message": f"{path} does not exist for patching"}
        original = "".join(read_lines(path))
        new_content, warnings = apply_patch(original, params["patch"])
        if warnings:
            return {"status": "error", "warnings": warnings}
        write_lines(path, [new_content])
    else:
        # Full replace mode
        content = params.get("content", "")
        path.parent.mkdir(parents=True, exist_ok=True)
        path.write_text(content, encoding="utf-8")
    return {"status": "ok"}


@register("list_dir")
async def list_dir(params: Dict[str, Any]) -> Dict[str, Any]:
    root = Path(params.get("path", ".")).expanduser().resolve()
    if not root.is_dir():
        return {"status": "error", "message": f"{root} is not a directory"}
    entries: List[Dict[str, Any]] = []
    for p in sorted(root.rglob("*") if params.get("recursive") else root.iterdir()):
        entries.append(
            {
                "path": str(p.relative_to(root)),
                "type": "dir" if p.is_dir() else "file",
            }
        )
    return {"status": "ok", "entries": entries}