"""
Git helper tools: status, diff, commit, checkout.
"""

from __future__ import annotations

import subprocess
from pathlib import Path
from typing import Any, Dict

from kimi_code.tools import register


def _run_git(cmd: list[str], cwd: Path) -> Dict[str, Any]:
    """Low-level helper to run git commands."""
    try:
        proc = subprocess.run(
            cmd,
            cwd=cwd,
            capture_output=True,
            text=True,
            check=True,
        )
        return {"status": "ok", "stdout": proc.stdout.strip(), "stderr": proc.stderr.strip()}
    except subprocess.CalledProcessError as exc:
        return {"status": "error", "stdout": exc.stdout, "stderr": exc.stderr}


@register("git_status")
async def git_status(params: Dict[str, Any]) -> Dict[str, Any]:
    cwd = Path(params.get("cwd", ".")).expanduser().resolve()
    return _run_git(["git", "status", "--porcelain=v1"], cwd=cwd)


@register("git_diff")
async def git_diff(params: Dict[str, Any]) -> Dict[str, Any]:
    cwd = Path(params.get("cwd", ".")).expanduser().resolve()
    return _run_git(["git", "diff"], cwd=cwd)


@register("git_commit")
async def git_commit(params: Dict[str, Any]) -> Dict[str, Any]:
    cwd = Path(params.get("cwd", ".")).expanduser().resolve()
    message = params["message"]
    return _run_git(["git", "commit", "-m", message], cwd=cwd)


@register("git_checkout")
async def git_checkout(params: Dict[str, Any]) -> Dict[str, Any]:
    cwd = Path(params.get("cwd", ".")).expanduser().resolve()
    ref = params["ref"]
    return _run_git(["git", "checkout", ref], cwd=cwd)


@register("git_branch")
async def git_branch(params: Dict[str, Any]) -> Dict[str, Any]:
    cwd = Path(params.get("cwd", ".")).expanduser().resolve()
    return _run_git(["git", "branch", "--show-current"], cwd=cwd)