"""
Minimal LSP helper tools.
Right now only:
  - get_diagnostics (requires a running LSP server)
  - format_file     (textDocument/formatting)

Future: full language-server orchestration.
"""

from __future__ import annotations

import json
import subprocess
import tempfile
from pathlib import Path
from typing import Any, Dict, List, Optional

from kimi_code.tools import register


# --------------------------------------------------------------------------- #
# Utilities
# --------------------------------------------------------------------------- #
def _run_lsp(
    language: str,
    command: List[str],
    workspace: Path,
    document_path: Path,
    method: str,
    params: Dict[str, Any],
) -> Dict[str, Any]:
    """
    Extremely thin LSP RPC helper:
    1. Spawns the server once.
    2. Sends initialize + textDocument/didOpen + requested method.
    3. Returns the last response or an error.
    """
    try:
        proc = subprocess.Popen(
            command,
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            text=True,
            cwd=workspace,
        )
    except FileNotFoundError as exc:
        return {"status": "error", "message": str(exc)}

    # Build simple JSON-RPC message
    def _rpc(msg: Dict[str, Any]) -> str:
        payload = json.dumps(msg, separators=(",", ":"))
        return f"Content-Length: {len(payload)}\r\n\r\n{payload}"

    # Initialize
    init_msg = {
        "jsonrpc": "2.0",
        "id": 1,
        "method": "initialize",
        "params": {
            "processId": None,
            "rootUri": workspace.as_uri(),
            "capabilities": {},
        },
    }
    proc.stdin.write(_rpc(init_msg))
    proc.stdin.flush()

    # Read until initialize response (ignore headers)
    def _read() -> Dict[str, Any]:
        headers = {}
        while True:
            line = proc.stdout.readline()
            if not line.strip():
                break
            if ":" in line:
                key, val = line.split(":", 1)
                headers[key.strip()] = val.strip()
        length = int(headers["Content-Length"])
        body = proc.stdout.read(length)
        return json.loads(body)

    _read()  # throw away initialize response

    # Open document
    text = document_path.read_text(encoding="utf-8")
    did_open = {
        "jsonrpc": "2.0",
        "method": "textDocument/didOpen",
        "params": {
            "textDocument": {
                "uri": document_path.as_uri(),
                "languageId": language,
                "version": 1,
                "text": text,
            }
        },
    }
    proc.stdin.write(_rpc(did_open))
    proc.stdin.flush()

    # Actual request
    request = {
        "jsonrpc": "2.0",
        "id": 2,
        "method": method,
        "params": params,
    }
    proc.stdin.write(_rpc(request))
    proc.stdin.flush()

    # Read response
    resp = _read()
    proc.stdin.close()
    proc.terminate()

    return {"status": "ok", "result": resp.get("result") or []}


# --------------------------------------------------------------------------- #
# Tools
# --------------------------------------------------------------------------- #
@register("lsp_diagnostics")
async def lsp_diagnostics(params: Dict[str, Any]) -> Dict[str, Any]:
    workspace = Path(params.get("cwd", ".")).expanduser().resolve()
    document_path = Path(params["path"]).expanduser().resolve()
    language = params.get("language", "python")
    command = params.get(
        "command", ["python", "-m", "pylsp"]  # default to pylsp
    )

    # wait for diagnostics (simplified polling)
    resp = _run_lsp(
        language=language,
        command=command,
        workspace=workspace,
        document_path=document_path,
        method="textDocument/diagnostic",
        params={"textDocument": {"uri": document_path.as_uri()}},
    )
    return resp


@register("lsp_format")
async def lsp_format(params: Dict[str, Any]) -> Dict[str, Any]:
    workspace = Path(params.get("cwd", ".")).expanduser().resolve()
    document_path = Path(params["path"]).expanduser().resolve()
    language = params.get("language", "python")
    command = params.get("command", ["python", "-m", "pylsp"])

    resp = _run_lsp(
        language=language,
        command=command,
        workspace=workspace,
        document_path=document_path,
        method="textDocument/formatting",
        params={
            "textDocument": {"uri": document_path.as_uri()},
            "options": {"tabSize": 4, "insertSpaces": True},
        },
    )
    if resp["status"] != "ok":
        return resp

    # Apply edits
    edits = resp["result"]
    if edits:
        text = document_path.read_text(encoding="utf-8")
        # naive: take the first (and usually only) edit list
        new_text = apply_text_edits(text, edits)
        document_path.write_text(new_text, encoding="utf-8")

    return {"status": "ok", "edits_applied": len(edits)}


# --------------------------------------------------------------------------- #
# Helpers
# --------------------------------------------------------------------------- #
def apply_text_edits(original: str, edits: List[Dict[str, Any]]) -> str:
    """
    Apply LSP TextEdit list to original text.
    Edits must be non-overlapping and ordered by start position descending.
    """
    lines = original.splitlines(keepends=True)
    for edit in sorted(
        edits,
        key=lambda e: (e["range"]["start"]["line"], e["range"]["start"]["character"]),
        reverse=True,
    ):
        start_line = edit["range"]["start"]["line"]
        start_char = edit["range"]["start"]["character"]
        end_line = edit["range"]["end"]["line"]
        end_char = edit["range"]["end"]["character"]

        if start_line == end_line:
            line = lines[start_line]
            new_line = line[:start_char] + edit["newText"] + line[end_char:]
            lines[start_line] = new_line
        else:
            # Multi-line edits: replace entire span (simplified)
            lines[start_line : end_line + 1] = [edit["newText"]]
    return "".join(lines)