"""
Shell tool: run arbitrary commands with timeout & capture (stdout, stderr, retcode).
"""

from __future__ import annotations

import asyncio
import shlex
from pathlib import Path
from typing import Any, Dict

import pexpect  # type: ignore

from kimi_code.tools import register


@register("shell")
async def shell(params: Dict[str, Any]) -> Dict[str, Any]:
    cmd: str = params["command"]
    cwd = Path(params.get("cwd", ".")).expanduser().resolve()

    # Simple safeguard: reject interactive commands
    banned = {"vim", "nano", "emacs", "ssh", "top", "htop"}
    tokens = shlex.split(cmd)
    if any(t in banned for t in tokens):
        return {
            "status": "error",
            "stdout": "",
            "stderr": f"Command '{cmd}' is on the deny-list.",
            "returncode": 126,
        }

    try:
        # pexpect.spawn runs in a thread via asyncio
        proc = pexpect.spawn(
            cmd,
            cwd=str(cwd),
            timeout=params.get("timeout", 30),
            encoding="utf-8",
            echo=False,
        )
        stdout_lines: list[str] = []
        stderr_lines: list[str] = []
        while True:
            try:
                idx = proc.expect(["\r\n", "\n", pexpect.EOF, pexpect.TIMEOUT])
                if idx == 0 or idx == 1:
                    stdout_lines.append(proc.before)
                elif idx == 2:
                    if proc.before:
                        stdout_lines.append(proc.before)
                    break
                elif idx == 3:
                    stderr_lines.append(proc.after or "")
            except pexpect.EOF:
                break

        proc.close()
        return {
            "status": "ok",
            "stdout": "\n".join(stdout_lines).strip(),
            "stderr": "\n".join(stderr_lines).strip(),
            "returncode": proc.exitstatus or 0,
        }
    except Exception as exc:
        return {
            "status": "error",
            "stdout": "",
            "stderr": str(exc),
            "returncode": 1,
        }